@@ -30,6 +30,27 @@
     (function-lambda-list #'(sb-pcl::slow-method cl-user::j (t)))
   (sb-pcl::method-args sb-pcl::next-methods))
 
+(deftest macro-lambda-list.1
+    (equal (function-lambda-list (defmacro macro-lambda-list.1-m (x b)
+                                   `(x b)))
+           '(x b))
+  t)
+
+#+sb-eval
+(deftest macro-lambda-list.2
+    (equal (function-lambda-list (interpret (defmacro macro-lambda-list.2-m (x)
+                                              x)))
+           '(x))
+  t)
+
+;; The CHECK-TYPE vop does not have a generator function. Make sure
+;; (find-definition-sources-by-name 'check-type :vop) returns NIL
+;; instead of signaling an error.
+(deftest definition-source.1
+    (values (find-definition-sources-by-name 'check-type :vop)
+            (listp (find-definition-sources-by-name 'check-type :macro)))
+  nil t)
+
 (deftest definition-source-plist.1
     (let* ((source (find-definition-source #'cl-user::one))
            (plist (definition-source-plist source))
