@@ -75,7 +75,7 @@
          (limit (truncate (- *print-right-margin* reserve) columns)))
     (flet ((trunc (&optional end)
              (let ((line-end (- limit 2)))
-               (with-output-to-string (s)
+               (with-simple-output-to-string (s)
                  (write-string line s :end (if end
                                                (min end line-end)
                                                line-end))
@@ -397,28 +397,24 @@
                    #'string<))
            (out (label list)
              (describe-stuff label list stream :escape nil)))
-      (let ((implemented (humanize (package-implemented-by-list package)))
-            (implements (humanize (package-implements-list package)))
-            (nicks (humanize (package-nicknames package)))
-            (uses (humanize (package-use-list package)))
-            (used (humanize (package-used-by-list package)))
-            (shadows (humanize (package-shadowing-symbols package)))
-            (this (list (package-name package)))
-            (exports nil))
+      (let ((exports nil))
         (do-external-symbols (ext package)
           (push ext exports))
-        (setf exports (humanize exports))
-        (when (package-locked-p package)
-          (format stream "~@:_Locked."))
-        (when (set-difference implemented this :test #'string=)
-          (out "Implemented-by-list" implemented))
-        (when (set-difference implements this :test #'string=)
-          (out "Implements-list" implements))
-        (out "Nicknames" nicks)
-        (out "Use-list" uses)
-        (out "Used-by-list" used)
-        (out "Shadows" shadows)
-        (out "Exports" exports)
+        #+sb-package-locks
+        (let ((implemented (humanize (package-implemented-by-list package)))
+              (implements (humanize (package-implements-list package)))
+              (this (list (package-name package))))
+          (when (package-locked-p package)
+            (format stream "~@:_Locked."))
+          (when (set-difference implemented this :test #'string=)
+            (out "Implemented-by-list" implemented))
+          (when (set-difference implements this :test #'string=)
+            (out "Implements-list" implements)))
+        (out "Nicknames" (humanize (package-nicknames package)))
+        (out "Use-list" (humanize (package-use-list package)))
+        (out "Used-by-list" (humanize (package-used-by-list package)))
+        (out "Shadows" (humanize (package-shadowing-symbols package)))
+        (out "Exports" (humanize exports))
         (format stream "~@:_~S internal symbols."
                 (package-internal-symbol-count package))))
     (terpri stream)))
