@@ -549,7 +549,7 @@
        (declare (optimize (speed 3)))
        (when (eql type instance-header-widetag)
          (incf total-objects)
-         (let* ((classoid (layout-classoid (%instance-ref obj 0)))
+         (let* ((classoid (layout-classoid (%instance-layout obj)))
                 (found (gethash classoid totals))
                 (size size))
            (declare (fixnum size))
@@ -745,10 +745,11 @@
                     (eq (cdr obj) object))
             (maybe-call fun obj)))
          (instance
-          (do-instance-tagged-slot (i obj)
-            (when (eq (%instance-ref obj i) object)
-              (maybe-call fun obj)
-              (return))))
+          (when (or (eq (%instance-layout obj) object)
+                    (do-instance-tagged-slot (i obj)
+                      (when (eq (%instance-ref obj i) object)
+                        (return t))))
+            (maybe-call fun obj)))
          (code-component
           (let ((length (get-header-data obj)))
             (do ((i code-constants-offset (1+ i)))
