@@ -112,6 +112,7 @@ SYSCALL-FORM. Repeat evaluation of SYSCALL-FORM if it is interrupted."
 
 #!-win32
 (define-alien-routine ("getenv" posix-getenv) c-string
+  #!+sb-doc
   "Return the \"value\" part of the environment string \"name=value\" which
 corresponds to NAME, or NIL if there is none."
   (name (c-string :not-null t)))
@@ -273,6 +274,7 @@ corresponds to NAME, or NIL if there is none."
   #!+win32 (sb!win32::windows-isatty fd))
 
 (defun unix-lseek (fd offset whence)
+  #!+sb-doc
   "Unix-lseek accepts a file descriptor and moves the file pointer by
    OFFSET octets.  Whence can be any of the following:
 
@@ -385,17 +387,25 @@ corresponds to NAME, or NIL if there is none."
   ;;
   ;; Signal an error at compile-time, since it's needed for the
   ;; runtime to start up
-  #!-(or linux openbsd freebsd netbsd sunos osf1 darwin hpux win32)
+  #!-(or android linux openbsd freebsd netbsd sunos osf1 darwin hpux win32 dragonfly)
   #.(error "POSIX-GETCWD is not implemented.")
-  #!+(or linux openbsd freebsd netbsd sunos osf1 darwin hpux win32)
-  (or (newcharstar-string (alien-funcall (extern-alien "getcwd"
-                                                       (function (* char)
-                                                                 (* char)
-                                                                 size-t))
-                                         nil
-                                         #!+(or linux openbsd freebsd netbsd darwin win32) 0
-                                         #!+(or sunos osf1 hpux) 1025))
-      (simple-perror "getcwd")))
+  (or
+   #!+(or linux openbsd freebsd netbsd sunos osf1 darwin hpux win32 dragonfly)
+   (newcharstar-string (alien-funcall (extern-alien "getcwd"
+                                                    (function (* char)
+                                                              (* char)
+                                                              size-t))
+                                      nil
+                                      #!+(or linux openbsd freebsd netbsd darwin win32 dragonfly) 0
+                                      #!+(or sunos osf1 hpux) 1025))
+   #!+android
+   (with-alien ((ptr (array char #.path-max)))
+     ;; Older bionic versions do not have the above feature.
+     (alien-funcall
+      (extern-alien "getcwd"
+                    (function c-string (array char #.path-max) int))
+      ptr path-max))
+   (simple-perror "getcwd")))
 
 ;;; Return the Unix current directory as a SIMPLE-STRING terminated
 ;;; by a slash character.
@@ -610,6 +620,15 @@ avoiding atexit(3) hooks, etc. Otherwise exit(2) is called."
               (events  short)           ; requested events
               (revents short)))         ; returned events
 
+  (defun unix-poll (pollfds nfds to-msec)
+    (declare (fixnum nfds to-msec))
+    (when (and (minusp to-msec) (not *interrupts-enabled*))
+      (note-dangerous-wait "poll(2)"))
+    ;; FAST-SELECT doesn't use WITH-RESTARTED-SYSCALL so this doesn't either
+    (int-syscall ("poll" (* (struct pollfd)) int int)
+                 (alien-sap pollfds) nfds to-msec))
+
+  ;; "simple" poll operates on a single descriptor only
   (defun unix-simple-poll (fd direction to-msec)
     (declare (fixnum fd to-msec))
     (when (and (minusp to-msec) (not *interrupts-enabled*))
@@ -807,7 +826,7 @@ avoiding atexit(3) hooks, etc. Otherwise exit(2) is called."
 (define-alien-type nil
   (struct wrapped_stat
     (st-dev wst-dev-t)
-    (st-ino ino-t)
+    (st-ino wst-ino-t)
     (st-mode mode-t)
     (st-nlink wst-nlink-t)
     (st-uid wst-uid-t)
@@ -991,7 +1010,8 @@ avoiding atexit(3) hooks, etc. Otherwise exit(2) is called."
 
 #!-win32
 (defun unix-getitimer (which)
-  "Unix-getitimer returns the INTERVAL and VALUE slots of one of
+  #!+sb-doc
+  "UNIX-GETITIMER returns the INTERVAL and VALUE slots of one of
    three system timers (:real :virtual or :profile). On success,
    unix-getitimer returns 5 values,
    T, it-interval-secs, it-interval-usec, it-value-secs, it-value-usec."
@@ -1014,7 +1034,8 @@ avoiding atexit(3) hooks, etc. Otherwise exit(2) is called."
 
 #!-win32
 (defun unix-setitimer (which int-secs int-usec val-secs val-usec)
-  " Unix-setitimer sets the INTERVAL and VALUE slots of one of
+  #!+sb-doc
+  "UNIX-SETITIMER sets the INTERVAL and VALUE slots of one of
    three system timers (:real :virtual or :profile). A SIGALRM signal
    will be delivered VALUE <seconds+microseconds> from now. INTERVAL,
    when non-zero, is <seconds+microseconds> to be loaded each time
@@ -1064,6 +1085,7 @@ avoiding atexit(3) hooks, etc. Otherwise exit(2) is called."
 
   #!-sb-fluid (declaim (inline get-time-of-day))
   (defun get-time-of-day ()
+    #!+sb-doc
     "Return the number of seconds and microseconds since the beginning of
 the UNIX epoch (January 1st 1970.)"
     #!+(or darwin netbsd)
