@@ -38,6 +38,11 @@
   ;; the EQ table.
   (equal-table (make-hash-table :test 'equal) :type hash-table)
   (eq-table (make-hash-table :test 'eq) :type hash-table)
+  ;; Hashtable mapping a string to a list of fop-table indices of
+  ;; symbols whose name is that string. For any name as compared
+  ;; by STRING= there can be a symbol whose name is a base string
+  ;; and/or a symbol whose name is not a base string.
+  (string=-table (make-hash-table :test 'equal) :type hash-table)
   ;; the table's current free pointer: the next offset to be used
   (table-free 0 :type index)
   ;; an alist (PACKAGE . OFFSET) of the table offsets for each package
@@ -167,16 +172,15 @@
 ;;; optimizations should be conditional on #!+SB-FROZEN.
 (defmacro dump-fop (fs-expr file &rest args)
   (let* ((fs (eval fs-expr))
-         (val (get fs 'opcode))
+         (val (or (get fs 'opcode)
+                  (error "compiler bug: ~S is not a legal fasload operator."
+                         fs-expr)))
          (fop-argc
-          (if val
-              (if (>= val +2-operand-fops+) 2 (sbit *fop-argp* (ash val -2)))
-              0))
-         (supplied-argc (length args)))
+          (if (>= val +2-operand-fops+)
+              2
+              (sbit (car *fop-signatures*) (ash val -2)))))
     (cond
-      ((not val)
-       (error "compiler bug: ~S is not a legal fasload operator." fs-expr))
-      ((not (eql supplied-argc fop-argc))
+      ((not (eql (length args) fop-argc))
        (error "~S takes ~D argument~:P" fs fop-argc))
       (t
       `(progn
@@ -934,6 +938,7 @@
   (declare (type fasl-output file))
   (let* ((pname (symbol-name s))
          (pname-length (length pname))
+         (dumped-as-copy nil)
          (pkg (symbol-package s)))
     ;; see comment in genesis: we need this here for repeatable fasls
     #+sb-xc-host
@@ -946,8 +951,21 @@
         (setq pkg sb!int:*cl-package*)))
 
     (cond ((null pkg)
-           (dump-fop 'fop-uninterned-symbol-save
-                     file pname-length))
+           (let ((this-base-p (typep pname 'base-string)))
+             (dolist (lookalike (gethash pname (fasl-output-string=-table file))
+                                (dump-fop 'fop-uninterned-symbol-save
+                                          file pname-length))
+               ;; Find the right kind of lookalike symbol.
+               ;; actually this seems pretty bogus - afaict, we don't correctly
+               ;; preserve the type of the string (base or character) anyway,
+               ;; but if we did, then this would be right also.
+               ;; [what about a symbol whose name is a (simple-array nil (0))?]
+               (let ((that-base-p (typep (symbol-name lookalike) 'base-string)))
+                 (when (or (and this-base-p that-base-p)
+                           (and (not this-base-p) (not that-base-p)))
+                   (dump-fop 'fop-copy-symbol-save file
+                             (gethash lookalike (fasl-output-eq-table file)))
+                   (return (setq dumped-as-copy t)))))))
           ;; CMU CL had FOP-SYMBOL-SAVE/FOP-SMALL-SYMBOL-SAVE fops which
           ;; used the current value of *PACKAGE*. Unfortunately that's
           ;; broken w.r.t. ANSI Common Lisp semantics, so those are gone
@@ -964,10 +982,12 @@
            (dump-fop 'fop-symbol-in-package-save file
                      (dump-package pkg file) pname-length)))
 
-    #+sb-xc-host (dump-base-chars-of-string pname file)
-    #-sb-xc-host (#!+sb-unicode dump-characters-of-string
-                  #!-sb-unicode dump-base-chars-of-string
-                  pname file)
+    (unless dumped-as-copy
+      #+sb-xc-host (dump-base-chars-of-string pname file)
+      #-sb-xc-host (#!+sb-unicode dump-characters-of-string
+                    #!-sb-unicode dump-base-chars-of-string
+                    pname file)
+      (push s (gethash (symbol-name s) (fasl-output-string=-table file))))
 
     (setf (gethash s (fasl-output-eq-table file))
           (fasl-output-table-free file))
@@ -1213,18 +1233,6 @@
   (dump-byte 0 fasl-output)
   (values))
 
-;;; Dump a FOP-FSET to arrange static linkage (at cold init) between
-;;; FUN-NAME and the already-dumped function whose dump handle is
-;;; FUN-DUMP-HANDLE.
-#+sb-xc-host
-(defun fasl-dump-cold-fset (fun-name fun-dump-handle fasl-output)
-  (declare (type fixnum fun-dump-handle))
-  (aver (legal-fun-name-p fun-name))
-  (dump-non-immediate-object fun-name fasl-output)
-  (dump-push fun-dump-handle fasl-output)
-  (dump-fop 'fop-fset fasl-output)
-  (values))
-
 ;;; Compute the correct list of DEBUG-SOURCE structures and backpatch
 ;;; all of the dumped DEBUG-INFO structures. We clear the
 ;;; FASL-OUTPUT-DEBUG-INFO, so that subsequent components with
@@ -1303,6 +1311,13 @@
     ;; Q: Shouldn't we aver that NAME is the proper name for its classoid?
     (unless name
       (compiler-error "dumping anonymous layout: ~S" obj))
+    ;; The target lisp can save some space in fasls (sometimes),
+    ;; but the cross-compiler can't because we need to construct the
+    ;; cold representation of all layouts, not reference host layouts.
+    #-sb-xc-host
+    (let ((fop (known-layout-fop name)))
+      (when fop
+        (return-from dump-layout (dump-byte fop file))))
     (dump-object name file))
   (sub-dump-object (layout-inherits obj) file)
   (sub-dump-object (layout-depthoid obj) file)
