@@ -53,8 +53,12 @@
   ;; Special forms which we don't currently handle, but might consider
   ;; supporting in the future are LOCALLY (with declarations),
   ;; MACROLET, SYMBOL-MACROLET and THE.
+  ;; Also, if (FLET ((F () ...)) (DEFUN A () ...) (DEFUN B () ...))
+  ;; were handled, then it would probably automatically work in
+  ;; the cold loader too, providing definitions for A and B before
+  ;; executing all other toplevel forms.
   #+sb-xc-host
-  (declare (ignore form expand))
+  (and expand (eq (car form) 'sb!impl::%defun))
   #-sb-xc-host
   (flet ((expand (form)
            (if expand
@@ -271,160 +275,178 @@
 ;;;
 ;;; See the expansion problem FIXME above fopcompilable-p.
 (defun fopcompile (form path for-value-p &optional (expand t))
-  (flet ((expand (form)
-           (if expand
-               (%macroexpand form *lexenv*)
-               (values form nil)))
-         (expand-cm (form)
-           (if expand
-               (expand-compiler-macro form)
-               (values form nil))))
-    (cond ((self-evaluating-p form)
-           (fopcompile-constant form for-value-p))
-          ((symbolp form)
-           (multiple-value-bind (macroexpansion macroexpanded-p)
-               (expand form)
-             (if macroexpanded-p
-                 ;; Symbol macro
-                 (fopcompile macroexpansion path for-value-p)
-                 (let ((kind (info :variable :kind form)))
-                   (cond
-                     ((eq :special kind)
-                      ;; Special variable
-                      (fopcompile `(symbol-value ',form) path for-value-p))
+  (let ((path (or (get-source-path form) (cons form path))))
+   (flet ((expand (form)
+            (if expand
+                (%macroexpand form *lexenv*)
+                (values form nil)))
+          (expand-cm (form)
+            (if expand
+                (expand-compiler-macro form)
+                (values form nil))))
+     (cond ((self-evaluating-p form)
+            (fopcompile-constant form for-value-p))
+           ((symbolp form)
+            (multiple-value-bind (macroexpansion macroexpanded-p)
+                (expand form)
+              (if macroexpanded-p
+                  ;; Symbol macro
+                  (fopcompile macroexpansion path for-value-p)
+                  (let ((kind (info :variable :kind form)))
+                    (cond
+                      ((eq :special kind)
+                       ;; Special variable
+                       (fopcompile `(symbol-value ',form) path for-value-p))
 
-                     ((member kind '(:global :constant))
-                      ;; Global variable or constant.
-                      (fopcompile `(symbol-global-value ',form) path for-value-p))
-                     (t
-                      ;; Lexical
-                      (let* ((lambda-var (cdr (assoc form (lexenv-vars *lexenv*))))
-                             (handle (when lambda-var
-                                       (lambda-var-fop-value lambda-var))))
-                        (if handle
-                            (when for-value-p
-                              (sb!fasl::dump-push handle *compile-object*))
-                            (progn
-                              ;; Undefined variable. Signal a warning, and
-                              ;; treat it as a special variable reference, like
-                              ;; the real compiler does -- do not elide even if
-                              ;; the value is unused.
-                              (note-undefined-reference form :variable)
-                              (fopcompile `(symbol-value ',form)
-                                          path
-                                          for-value-p))))))))))
-          ((listp form)
-           (let ((macroexpansion (expand-cm form)))
-             (if (neq macroexpansion form)
-                 ;; could expand into an atom, so start from the top
-                 (return-from fopcompile
-                   (fopcompile macroexpansion path for-value-p))))
-           (multiple-value-bind (macroexpansion macroexpanded-p)
-               (expand form)
-             (if macroexpanded-p
-                 (fopcompile macroexpansion path for-value-p)
-                 (destructuring-bind (operator &rest args) form
-                   (case operator
-                     ;; The QUOTE special operator is worth handling: very
-                     ;; easy and very common at toplevel.
-                     ((quote)
-                      (fopcompile-constant (second form) for-value-p))
-                     ;; A FUNCTION needs to be compiled properly, but doesn't
-                     ;; need to prevent the fopcompilation of the whole form.
-                     ;; We just compile it, and emit an instruction for pushing
-                     ;; the function handle on the FOP stack.
-                     ((function)
-                      (fopcompile-function (second form) path for-value-p))
-                     ;; KLUDGE! SB!C:SOURCE-LOCATION calls are normally handled
-                     ;; by a compiler-macro. But if SPACE > DEBUG we choose not
-                     ;; to record locations, which is strange because the main
-                     ;; compiler does not have similar logic afaict.
-                     ((source-location)
-                      (if (policy *policy* (and (> space 1)
-                                                (> space debug)))
-                          (fopcompile-constant nil for-value-p)
-                          (fopcompile (let ((*current-path* path))
-                                        (make-definition-source-location))
-                                      path
-                                      for-value-p)))
-                     ((if)
-                      (fopcompile-if args path for-value-p))
-                     ((progn locally)
-                      (loop for (arg . next) on args
-                            do (fopcompile arg
-                                           path (if next
-                                                    nil
-                                                    for-value-p))))
-                     ((setq)
-                      (loop for (name value . next) on args by #'cddr
-                            do (fopcompile `(set ',name ,value) path
-                                           (if next
-                                               nil
-                                               for-value-p))))
-                     ((eval-when)
-                      (destructuring-bind (situations &body body) args
-                        (if (or (member :execute situations)
-                                (member 'eval situations))
-                            (fopcompile (cons 'progn body) path for-value-p)
-                            (fopcompile nil path for-value-p))))
-                     ((let let*)
-                      (let ((orig-lexenv *lexenv*)
-                            (*lexenv* (make-lexenv :default *lexenv*)))
-                        (loop for binding in (car args)
-                              for name = (if (consp binding)
-                                             (first binding)
-                                             binding)
-                              for value = (if (consp binding)
-                                              (second binding)
-                                              nil)
-                              do (let ((*lexenv* (if (eql operator 'let)
-                                                     orig-lexenv
-                                                     *lexenv*)))
-                                   (fopcompile value path t))
-                              do (let ((obj (sb!fasl::dump-pop *compile-object*)))
-                                   (setf *lexenv*
-                                         (make-lexenv
-                                          :vars (list (cons name
-                                                            (make-lambda-var
-                                                             :%source-name name
-                                                             :fop-value obj)))))))
-                        (fopcompile (cons 'progn (cdr args)) path for-value-p)))
-                     ;; Otherwise it must be an ordinary funcall.
-                     (otherwise
-                      (cond
-                        ;; Special hack: there's already a fop for
-                        ;; find-undeleted-package-or-lose, so use it.
-                        ;; (We could theoretically do the same for
-                        ;; other operations, but I don't see any good
-                        ;; candidates in a quick read-through of
-                        ;; src/code/fop.lisp.)
-                        ((and (eq operator
-                                  'sb!int:find-undeleted-package-or-lose)
-                              (= 1 (length args))
-                              for-value-p)
-                         (fopcompile (first args) path t)
-                         (sb!fasl::dump-fop 'sb!fasl::fop-package
-                                            *compile-object*))
-                        (t
-                         (when (eq (info :function :where-from operator) :assumed)
-                           (note-undefined-reference operator :function))
-                         (fopcompile-constant operator t)
-                         (dolist (arg args)
-                           (fopcompile arg path t))
-                         (if for-value-p
-                             (sb!fasl::dump-fop 'sb!fasl::fop-funcall
-                                                *compile-object*)
-                             (sb!fasl::dump-fop 'sb!fasl::fop-funcall-for-effect
-                                                *compile-object*))
-                         (let ((n-args (length args)))
-                           ;; stub: FOP-FUNCALL isn't going to be usable
-                           ;; to compile more than this, since its count
-                           ;; is a single byte. Maybe we should just punt
-                           ;; to the ordinary compiler in that case?
-                           (aver (<= n-args 255))
-                           (sb!fasl::dump-byte n-args *compile-object*))))))))))
-          (t
-           (bug "looks unFOPCOMPILEable: ~S" form)))))
+                      ((member kind '(:global :constant))
+                       ;; Global variable or constant.
+                       (fopcompile `(symbol-global-value ',form) path for-value-p))
+                      (t
+                       ;; Lexical
+                       (let* ((lambda-var (cdr (assoc form (lexenv-vars *lexenv*))))
+                              (handle (when lambda-var
+                                        (lambda-var-fop-value lambda-var))))
+                         (cond (handle
+                                (setf (lambda-var-ever-used lambda-var) t)
+                                (when for-value-p
+                                  (sb!fasl::dump-push handle *compile-object*)))
+                               (t
+                                ;; Undefined variable. Signal a warning, and
+                                ;; treat it as a special variable reference, like
+                                ;; the real compiler does -- do not elide even if
+                                ;; the value is unused.
+                                (note-undefined-reference form :variable)
+                                (fopcompile `(symbol-value ',form)
+                                            path
+                                            for-value-p))))))))))
+           ((listp form)
+            (let ((macroexpansion (expand-cm form)))
+              (if (neq macroexpansion form)
+                  ;; could expand into an atom, so start from the top
+                  (return-from fopcompile
+                    (fopcompile macroexpansion path for-value-p))))
+            (multiple-value-bind (macroexpansion macroexpanded-p)
+                (expand form)
+              (if macroexpanded-p
+                  (fopcompile macroexpansion path for-value-p)
+                  (destructuring-bind (operator &rest args) form
+                    (case operator
+                      ;; The QUOTE special operator is worth handling: very
+                      ;; easy and very common at toplevel.
+                      ((quote)
+                       (fopcompile-constant (second form) for-value-p))
+                      ;; A FUNCTION needs to be compiled properly, but doesn't
+                      ;; need to prevent the fopcompilation of the whole form.
+                      ;; We just compile it, and emit an instruction for pushing
+                      ;; the function handle on the FOP stack.
+                      ((function)
+                       (fopcompile-function (second form) path for-value-p))
+                      ;; KLUDGE! SB!C:SOURCE-LOCATION calls are normally handled
+                      ;; by a compiler-macro. But if SPACE > DEBUG we choose not
+                      ;; to record locations, which is strange because the main
+                      ;; compiler does not have similar logic afaict.
+                      ((source-location)
+                       (if (policy *policy* (and (> space 1)
+                                                 (> space debug)))
+                           (fopcompile-constant nil for-value-p)
+                           (fopcompile (let ((*current-path* path))
+                                         (make-definition-source-location))
+                                       path
+                                       for-value-p)))
+                      ((if)
+                       (fopcompile-if args path for-value-p))
+                      ((progn locally)
+                       (loop for (arg . next) on args
+                             do (fopcompile arg
+                                            path (if next
+                                                     nil
+                                                     for-value-p))))
+                      ((setq)
+                       (loop for (name value . next) on args by #'cddr
+                             do (fopcompile `(set ',name ,value) path
+                                            (if next
+                                                nil
+                                                for-value-p))))
+                      ((eval-when)
+                       (destructuring-bind (situations &body body) args
+                         (if (or (member :execute situations)
+                                 (member 'eval situations))
+                             (fopcompile (cons 'progn body) path for-value-p)
+                             (fopcompile nil path for-value-p))))
+                      ((let let*)
+                       (let ((orig-lexenv *lexenv*)
+                             (*lexenv* (make-lexenv :default *lexenv*))
+                             vars)
+                         (loop for binding in (car args)
+                               for name = (if (consp binding)
+                                              (first binding)
+                                              binding)
+                               for value = (if (consp binding)
+                                               (second binding)
+                                               nil)
+                               do
+                               (let ((*lexenv* (if (eql operator 'let)
+                                                   orig-lexenv
+                                                   *lexenv*)))
+                                 (fopcompile value path t))
+                               (let* ((obj (sb!fasl::dump-pop *compile-object*))
+                                      (var (make-lambda-var
+                                            :%source-name name
+                                            :fop-value obj)))
+                                 (push var vars)
+                                 (setf *lexenv*
+                                       (make-lexenv
+                                        :vars (list (cons name var))))))
+                         (fopcompile (cons 'progn (cdr args)) path for-value-p)
+                         (when (and vars
+                                    (and *source-info* path))
+                           (let* ((tlf (source-path-tlf-number path))
+                                  (file-info (source-info-file-info *source-info*))
+                                  (*compiler-error-context*
+                                    (make-compiler-error-context
+                                     :original-source (stringify-form form)
+                                     :file-name (file-info-name file-info)
+                                     :file-position
+                                     (nth-value 1 (find-source-root tlf *source-info*))
+                                     :original-source-path (source-path-original-source path)
+                                     :lexenv *lexenv*)))
+                             (note-unreferenced-vars vars *policy*)))))
+                      ;; Otherwise it must be an ordinary funcall.
+                      (otherwise
+                       (cond
+                         ;; Special hack: there's already a fop for
+                         ;; find-undeleted-package-or-lose, so use it.
+                         ;; (We could theoretically do the same for
+                         ;; other operations, but I don't see any good
+                         ;; candidates in a quick read-through of
+                         ;; src/code/fop.lisp.)
+                         ((and (eq operator
+                                   'sb!int:find-undeleted-package-or-lose)
+                               (= 1 (length args))
+                               for-value-p)
+                          (fopcompile (first args) path t)
+                          (sb!fasl::dump-fop 'sb!fasl::fop-package
+                                             *compile-object*))
+                         (t
+                          (when (eq (info :function :where-from operator) :assumed)
+                            (note-undefined-reference operator :function))
+                          (fopcompile-constant operator t)
+                          (dolist (arg args)
+                            (fopcompile arg path t))
+                          (if for-value-p
+                              (sb!fasl::dump-fop 'sb!fasl::fop-funcall
+                                                 *compile-object*)
+                              (sb!fasl::dump-fop 'sb!fasl::fop-funcall-for-effect
+                                                 *compile-object*))
+                          (let ((n-args (length args)))
+                            ;; stub: FOP-FUNCALL isn't going to be usable
+                            ;; to compile more than this, since its count
+                            ;; is a single byte. Maybe we should just punt
+                            ;; to the ordinary compiler in that case?
+                            (aver (<= n-args 255))
+                            (sb!fasl::dump-byte n-args *compile-object*))))))))))
+           (t
+            (bug "looks unFOPCOMPILEable: ~S" form))))))
 
 (defun fopcompile-function (form path for-value-p)
   (flet ((dump-fdefinition (name)
