@@ -121,7 +121,8 @@
 ;;; compiler as we can make it, i.e. identical in most ways, including
 ;;; this one. -- WHN 2001-08-19
 (declaim (type (simple-vector #.(ash 1 type-number-bits)) *info-types*))
-(defglobal *info-types* (make-array (ash 1 type-number-bits) :initial-element nil))
+(!defglobal *info-types*
+            (make-array (ash 1 type-number-bits) :initial-element nil))
 
 (defstruct (type-info
             #-no-ansi-print-object
@@ -560,8 +561,11 @@
 
 ;;; a macro-like function which transforms a call to this function
 ;;; into some other Lisp form. This expansion is inhibited if inline
-;;; expansion is inhibited
-(define-info-type (:function :source-transform) :type-spec (or function null))
+;;; expansion is inhibited.
+;;; As an exception, a cons of two atoms represents structure metadata
+;;; which is recognized and transformed in a stylized way.
+(define-info-type (:function :source-transform)
+  :type-spec (or function null (cons atom atom)))
 
 ;;; the macroexpansion function for this macro
 (define-info-type (:function :macro-function) :type-spec (or function null))
@@ -577,8 +581,6 @@
 ;;; structure containing the info used to special-case compilation.
 (define-info-type (:function :info) :type-spec (or fun-info null))
 
-(define-info-type (:function :structure-accessor)
-  :type-spec (or defstruct-description null))
 
 ;;;; ":VARIABLE" subsection - Data pertaining to globally known variables.
 
@@ -794,9 +796,6 @@
 ;;; load time to the same state they have currently.
 (!cold-init-forms
   (/show0 "beginning *INFO-TYPES* initialization")
-  #-sb-xc-host
-  ;; Host already has this array, do not clobber it
-  (setq *info-types* (make-array (ash 1 type-number-bits) :initial-element nil))
   (mapc (lambda (x)
           (register-info-metadata (first x) (second x) (third x) (fourth x)))
         '#.(loop for info-type across *info-types*
