@@ -1227,12 +1227,12 @@
   (declare (type cblock block1 block2))
   (or (eq block1 block2)
       (let ((cleanup2 (block-start-cleanup block2)))
-        (do ((cleanup (block-end-cleanup block1)
-                      (node-enclosing-cleanup (cleanup-mess-up cleanup))))
-            ((eq cleanup cleanup2) t)
+        (do-nested-cleanups (cleanup block1 t)
+          (when (eq cleanup cleanup2)
+            (return t))
           (case (cleanup-kind cleanup)
             ((:block :tagbody)
-             (unless (null (entry-exits (cleanup-mess-up cleanup)))
+             (when (entry-exits (cleanup-mess-up cleanup))
                (return nil)))
             (t (return nil)))))))
 
