@@ -547,6 +547,7 @@ necessary, since type inference may take arbitrarily long to converge.")
     (when (or (ir2-component-values-receivers (component-info component))
               (component-dx-lvars component))
       (maybe-mumble "stack ")
+      (find-dominators component)
       (stack-analyze component)
       ;; Assign BLOCK-NUMBER for any cleanup blocks introduced by
       ;; stack analysis. There shouldn't be any unreachable code after
@@ -832,6 +833,9 @@ necessary, since type inference may take arbitrarily long to converge.")
   ;; the stream that we are using to read the FILE-INFO, or NIL if
   ;; no stream has been opened yet
   (stream nil :type (or stream null))
+  ;; for coalescing DEFINITION-SOURCE-LOCATION of effectively toplevel forms
+  ;; inside one truly toplevel form.
+  (last-defn-source-loc)
   ;; if the current compilation is recursive (e.g., due to EVAL-WHEN
   ;; processing at compile-time), the invoking compilation's
   ;; source-info.
@@ -1256,19 +1260,6 @@ necessary, since type inference may take arbitrarily long to converge.")
 
               (mapc #'clear-ir1-info components-from-dfo))))))))
 
-#+sb-xc-host
-(defun process-toplevel-cold-fset (name lambda-expression path)
-  (unless (producing-fasl-file)
-    (error "can't COLD-FSET except in a fasl file"))
-  (legal-fun-name-or-type-error name)
-  (fasl-dump-cold-fset name
-                       (%compile lambda-expression
-                                 *compile-object*
-                                 :name name
-                                 :path path)
-                       *compile-object*)
-  (values))
-
 (defun note-top-level-form (form &optional finalp)
   (when *compile-print*
     (cond ((not *top-level-form-noted*)
@@ -1414,16 +1405,6 @@ necessary, since type inference may take arbitrarily long to converge.")
                                        (car form)
                                        form))))
               (case (car form)
-                ;; In the cross-compiler, top level COLD-FSET arranges
-                ;; for static linking at cold init time.
-                #+sb-xc-host
-                ((!cold-fset)
-                 (aver (not compile-time-too))
-                 (destructuring-bind (cold-fset fun-name lambda-expression) form
-                   (declare (ignore cold-fset))
-                   (process-toplevel-cold-fset fun-name
-                                               lambda-expression
-                                               path)))
                 ((eval-when macrolet symbol-macrolet);things w/ 1 arg before body
                  (need-at-least-one-arg form)
                  (destructuring-bind (special-operator magic &rest body) form
