@@ -383,12 +383,10 @@
              (lambda (offset)
                (make-random-tn :kind :normal
                                :sc (sc-or-lose sc-name)
-                               :offset offset)))
-           (out-of-registers-error ()
-             (error "Too many arguments in callback")))
+                               :offset offset))))
     (let* ((segment (make-segment))
            (rax rax-tn)
-           #!+(or win32 (not sb-safepoint)) (rcx rcx-tn)
+           #!+(or win32 (not sb-thread)) (rcx rcx-tn)
            #!-win32 (rdi rdi-tn)
            #!-win32 (rsi rsi-tn)
            (rdx rdx-tn)
@@ -455,7 +453,7 @@
                   (t
                    (bug "Unknown alien floating point type: ~S" type)))))
 
-        #!-sb-safepoint
+        #!-sb-thread
         (progn
           ;; arg0 to FUNCALL3 (function)
           ;;
@@ -488,7 +486,7 @@
           (inst mov rsp rbp)
           (inst pop rbp))
 
-        #!+sb-safepoint
+        #!+sb-thread
         (progn
           ;; arg0 to ENTER-ALIEN-CALLBACK (trampoline index)
           (inst mov #!-win32 rdi #!+win32 rcx (fixnumize index))
@@ -522,7 +520,7 @@
            (inst movq xmm0 [rsp]))
           ((alien-void-type-p result-type))
           (t
-           (error "unrecognized alien type: ~A" result-type)))
+           (error "Unrecognized alien type: ~A" result-type)))
 
         ;; Pop the arguments and the return value from the stack to get
         ;; the return address at top of stack.
