@@ -22,6 +22,16 @@
                      (nth n *register-arg-offsets*))
       (make-wired-tn *backend-t-primitive-type* control-stack-sc-number n)))
 
+(defun standard-arg-location-sc (n)
+  (declare (type unsigned-byte n))
+  (if (< n register-arg-count)
+      (make-sc-offset descriptor-reg-sc-number
+                      (nth n *register-arg-offsets*))
+      (make-sc-offset control-stack-sc-number n)))
+
+(defconstant arg-count-sc (make-sc-offset any-reg-sc-number ecx-offset))
+(defconstant closure-sc (make-sc-offset descriptor-reg-sc-number eax-offset))
+
 ;;; Make a passing location TN for a local call return PC.
 ;;;
 ;;; Always wire the return PC location to the stack in its standard
@@ -271,11 +281,9 @@
                               ancestor-frame-set/system-area-pointer))))
 
 (define-vop (xep-allocate-frame)
-  (:info start-lab copy-more-arg-follows)
-  (:vop-var vop)
+  (:info start-lab)
   (:generator 1
     (emit-alignment n-lowtag-bits)
-    (trace-table-entry trace-table-fun-prologue)
     (emit-label start-lab)
     ;; Skip space for the function header.
     (inst simple-fun-header-word)
@@ -284,20 +292,15 @@
 
     ;; The start of the actual code.
     ;; Save the return-pc.
-    (popw ebp-tn (frame-word-offset return-pc-save-offset))
+    (popw ebp-tn (frame-word-offset return-pc-save-offset))))
 
-    ;; If copy-more-arg follows it will allocate the correct stack
-    ;; size. The stack is not allocated first here as this may expose
-    ;; args on the stack if they take up more space than the frame!
-    (unless copy-more-arg-follows
-      ;; The args fit within the frame so just allocate the frame.
-      (inst lea esp-tn
+(define-vop (xep-setup-sp)
+  (:generator 1
+    (inst lea esp-tn
             (make-ea :dword :base ebp-tn
                      :disp (- (* n-word-bytes
                                  (- (max 3 (sb-allocated-size 'stack))
-                                    sp->fp-offset))))))
-
-    (trace-table-entry trace-table-normal)))
+                                    sp->fp-offset)))))))
 
 ;;; This is emitted directly before either a known-call-local, call-local,
 ;;; or a multiple-call-local. All it does is allocate stack space for the
@@ -437,7 +440,6 @@
            (let ((defaults (defaults)))
              (when defaults
                (assemble (*elsewhere*)
-                 (trace-table-entry trace-table-fun-prologue)
                  (emit-label default-stack-slots)
                  (dolist (default defaults)
                    (emit-label (car default))
@@ -449,8 +451,7 @@
                      ;; above.
                      (inst push edx-tn))
                    (inst mov (second default) eax-tn))
-                 (inst jmp defaulting-done)
-                 (trace-table-entry trace-table-normal)))))))
+                 (inst jmp defaulting-done)))))))
       (t
        ;; 91 bytes for this branch.
        (let ((regs-defaulted (gen-label))
@@ -664,12 +665,10 @@
   (:ignore nfp arg-locs args callee)
   (:node-var node)
   (:generator 5
-    (trace-table-entry trace-table-call-site)
     (move ebp-tn fp)
     (note-this-location vop :call-site)
     (inst call target)
-    (default-unknown-values vop values nvals node)
-    (trace-table-entry trace-table-normal)))
+    (default-unknown-values vop values nvals node)))
 
 ;;; Non-TR local call for a variable number of return values passed according
 ;;; to the unknown values convention. The results are the start of the values
@@ -685,13 +684,11 @@
   (:vop-var vop)
   (:node-var node)
   (:generator 20
-    (trace-table-entry trace-table-call-site)
     (move ebp-tn fp)
     (note-this-location vop :call-site)
     (inst call target)
     (note-this-location vop :unknown-return)
-    (receive-unknown-values values-start nvals start count node)
-    (trace-table-entry trace-table-normal)))
+    (receive-unknown-values values-start nvals start count node)))
 
 ;;;; local call with known values return
 
@@ -712,12 +709,10 @@
   (:ignore args res save nfp callee)
   (:vop-var vop)
   (:generator 5
-    (trace-table-entry trace-table-call-site)
     (move ebp-tn fp)
     (note-this-location vop :call-site)
     (inst call target)
-    (note-this-location vop :known-return)
-    (trace-table-entry trace-table-normal)))
+    (note-this-location vop :known-return)))
 
 ;;; From Douglas Crosher
 ;;; Return from known values call. We receive the return locations as
@@ -733,12 +728,10 @@
   (:vop-var vop)
   (:generator 6
     (check-ocfp-and-return-pc old-fp return-pc)
-    (trace-table-entry trace-table-fun-epilogue)
     ;; Zot all of the stack except for the old-fp and return-pc.
     (inst mov esp-tn ebp-tn)
     (inst pop ebp-tn)
-    (inst ret)
-    (trace-table-entry trace-table-normal)))
+    (inst ret)))
 
 ;;;; full call
 ;;;
@@ -857,7 +850,6 @@
                                (if (eq return :tail) 0 10)
                                15
                                (if (eq return :unknown) 25 0))
-               (trace-table-entry trace-table-call-site)
 
                ;; This has to be done before the frame pointer is
                ;; changed! EAX stores the 'lexical environment' needed
@@ -971,8 +963,7 @@
                     '((note-this-location vop :unknown-return)
                       (receive-unknown-values values-start nvals start count
                                               node)))
-                   (:tail))
-               (trace-table-entry trace-table-normal)))))
+                   (:tail))))))
 
   (define-full-call call nil :fixed nil)
   (define-full-call call-named t :fixed nil)
@@ -1020,7 +1011,6 @@
   (:ignore value)
   (:generator 6
     (check-ocfp-and-return-pc old-fp return-pc)
-    (trace-table-entry trace-table-fun-epilogue)
     ;; Drop stack above old-fp
     (inst mov esp-tn ebp-tn)
     ;; Clear the multiple-value return flag
@@ -1062,7 +1052,6 @@
     (when (= nvals 1)
       ;; This is handled in RETURN-SINGLE.
       (error "nvalues is 1"))
-    (trace-table-entry trace-table-fun-epilogue)
     ;; Establish the values pointer and values count.
     (inst lea ebx (make-ea :dword :base ebp-tn
                            :disp (* sp->fp-offset n-word-bytes)))
@@ -1100,9 +1089,7 @@
                                :disp (frame-byte-offset
                                       (+ sp->fp-offset
                                          (tn-offset return-pc)))))
-           (inst ret)))
-
-    (trace-table-entry trace-table-normal)))
+           (inst ret)))))
 
 ;;; Do unknown-values return of an arbitrary number of values (passed
 ;;; on the stack.) We check for the common case of a single return
@@ -1125,7 +1112,6 @@
   (:node-var node)
   (:generator 13
     (check-ocfp-and-return-pc old-fp return-pc)
-    (trace-table-entry trace-table-fun-epilogue)
     (unless (policy node (> space speed))
       ;; Check for the single case.
       (let ((not-single (gen-label)))
@@ -1144,19 +1130,10 @@
         (emit-label not-single)))
     (move esi vals)
     (move ecx nvals)
-    (inst jmp (make-fixup 'return-multiple :assembly-routine))
-    (trace-table-entry trace-table-normal)))
+    (inst jmp (make-fixup 'return-multiple :assembly-routine))))
 
 ;;;; XEP hackery
 
-;;; We don't need to do anything special for regular functions.
-(define-vop (setup-environment)
-  (:info label)
-  (:ignore label)
-  (:generator 0
-    ;; Don't bother doing anything.
-    nil))
-
 ;;; Get the lexical environment from its passing location.
 (define-vop (setup-closure-environment)
   (:results (closure :scs (descriptor-reg)))
@@ -1448,19 +1425,28 @@
 ;;; Signal wrong argument count error if NARGS isn't equal to COUNT.
 (define-vop (verify-arg-count)
   (:policy :fast-safe)
-  (:translate sb!c::%verify-arg-count)
   (:args (nargs :scs (any-reg)))
-  (:arg-types positive-fixnum (:constant t))
-  (:info count)
+  (:arg-types positive-fixnum (:constant t) (:constant t))
+  (:info min max)
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 3
     (let ((err-lab
            (generate-error-code vop 'invalid-arg-count-error nargs)))
-      (if (zerop count)
-          (inst test nargs nargs)  ; smaller instruction
-        (inst cmp nargs (fixnumize count)))
-      (inst jmp :ne err-lab))))
+      (cond ((not min)
+             (if (zerop max)
+                 (inst test nargs nargs)
+                 (inst cmp nargs (fixnumize max)))
+             (inst jmp :ne err-lab))
+            (max
+             (when (plusp min)
+               (inst cmp nargs (fixnumize min))
+               (inst jmp :b err-lab))
+             (inst cmp nargs (fixnumize max))
+             (inst jmp :a err-lab))
+            ((plusp min)
+             (inst cmp nargs (fixnumize min))
+             (inst jmp :b err-lab))))))
 
 ;;; Various other error signallers.
 (macrolet ((def (name error translate &rest args)
@@ -1476,7 +1462,7 @@
                 (:generator 1000
                   (error-call vop ',error ,@args)))))
   (def arg-count-error invalid-arg-count-error
-    sb!c::%arg-count-error nargs)
+    sb!c::%arg-count-error nargs fname)
   (def type-check-error object-not-type-error sb!c::%type-check-error
     object type)
   (def layout-invalid-error layout-invalid-error sb!c::%layout-invalid-error
