@@ -58,8 +58,7 @@
 #define LOCK_CREATE_THREAD
 #endif
 
-#ifdef LISP_FEATURE_FREEBSD
-#define CREATE_CLEANUP_THREAD
+#if defined(LISP_FEATURE_FREEBSD) || defined(LISP_FEATURE_DRAGONFLY)
 #define LOCK_CREATE_THREAD
 #endif
 
@@ -496,7 +495,6 @@ new_thread_trampoline(struct thread *th)
     return result;
 }
 
-# ifdef LISP_FEATURE_SB_SAFEPOINT
 static struct thread *create_thread_struct(lispobj);
 
 void
@@ -516,12 +514,18 @@ attach_os_thread(init_thread_data *scribble)
 #ifndef LISP_FEATURE_WIN32
     /* On windows, arch_os_thread_init will take care of finding the
      * stack. */
+    void *stack_addr;
+    size_t stack_size;
+#ifdef LISP_FEATURE_DARWIN
+    stack_addr = pthread_get_stackaddr_np(os);
+    stack_size = pthread_get_stacksize_np(os);
+#else
     pthread_attr_t attr;
     int pthread_getattr_np(pthread_t, pthread_attr_t *);
     pthread_getattr_np(os, &attr);
-    void *stack_addr;
-    size_t stack_size;
     pthread_attr_getstack(&attr, &stack_addr, &stack_size);
+#endif
+
     th->control_stack_start = stack_addr;
     th->control_stack_end = (void *) (((uintptr_t) stack_addr) + stack_size);
 #endif
@@ -555,8 +559,45 @@ detach_os_thread(init_thread_data *scribble)
     pthread_setspecific(lisp_thread, (void *)0);
     thread_sigmask(SIG_SETMASK, &scribble->oldset, 0);
 }
-# endif /* safepoint */
 
+void
+callback_wrapper_trampoline(
+#if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
+    /* On the x86oid backends, the assembly wrapper happens to not pass
+     * in ENTER_ALIEN_CALLBACK explicitly for safepoints.  However, the
+     * platforms with precise GC are tricky enough already, and I want
+     * to minimize the read-time conditionals.  For those platforms, I'm
+     * only replacing funcall3 with callback_wrapper_trampoline while
+     * keeping the arguments unchanged. --DFL */
+    lispobj __attribute__((__unused__)) fun,
+#endif
+    lispobj arg0, lispobj arg1, lispobj arg2)
+{
+#if defined(LISP_FEATURE_WIN32)
+    pthread_np_notice_thread();
+#endif
+    struct thread* th = arch_os_get_current_thread();
+    if (!th) {                  /* callback invoked in non-lisp thread */
+        init_thread_data scribble;
+        attach_os_thread(&scribble);
+        funcall3(StaticSymbolFunction(ENTER_FOREIGN_CALLBACK), arg0,arg1,arg2);
+        detach_os_thread(&scribble);
+        return;
+    }
+
+#ifdef LISP_FEATURE_WIN32
+    /* arg2 is the pointer to a return value, which sits on the stack */
+    th->carried_base_pointer = (os_context_register_t) *(((void**)arg2)-1);
+#endif
+
+#ifdef LISP_FEATURE_SB_SAFEPOINT
+    WITH_GC_AT_SAFEPOINTS_ONLY()
+#endif
+    {
+       funcall3(SymbolValue(ENTER_ALIEN_CALLBACK, 0), arg0, arg1, arg2);
+
+    }
+}
 #endif /* LISP_FEATURE_SB_THREAD */
 
 static void
@@ -712,6 +753,10 @@ create_thread_struct(lispobj initial_function) {
     SetSymbolValue(ALIEN_STACK_POINTER,(lispobj)th->alien_stack_pointer,th);
     SetSymbolValue(PSEUDO_ATOMIC_BITS,(lispobj)th->pseudo_atomic_bits,th);
 #endif
+#ifdef PSEUDO_ATOMIC_INTERRUPTED
+    SetSymbolValue(PSEUDO_ATOMIC_ATOMIC,NIL,th);
+    SetSymbolValue(PSEUDO_ATOMIC_INTERRUPTED,NIL,th);
+#endif
 #endif
     bind_variable(CURRENT_CATCH_BLOCK,make_fixnum(0),th);
     bind_variable(CURRENT_UNWIND_PROTECT_BLOCK,make_fixnum(0),th);
